use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
@DAGOLDEN
@DAGOLDENs
@ETHER
@ETHERs
@KENTNL
@KENTNLs
@TABULO
@TABULON
@TABULONs
@TABULOes
AllowOverride
AllowOverrides
AppendPrepend
AppendPrepends
ArchiveRelease
ArchiveReleases
AutoPrereqs
AutoPrereqses
AutoVersion
AutoVersions
BeforeRelease
BeforeReleases
BumpVersionAfterRelease
BumpVersionAfterReleases
CPAN
CPANS
CommentString
CommentStrings
ConfirmRelease
ConfirmReleases
DAGOLDEN
DAGOLDENS
ETHER
ETHERS
EnsureUniqueSections
EnsureUniqueSectionses
ExecDir
ExecDirs
Exporter
FakeRelease
FakeReleases
GatherDir
GatherDirs
GenerateSection
GenerateSections
GitHub
GitHubs
GithubMeta
GithubMetas
Handy
KENTNL
KENTNLS
MAXWORDLENGTH
MAXWORDLENGTHS
ManifestSkip
ManifestSkips
MetaJSON
MetaJSONs
MetaNoIndex
MetaNoIndexes
MetaResources
MetaResourceses
MetaYAML
MetaYAMLs
ModuleBuild
ModuleBuildTiny
ModuleBuildTinys
ModuleBuilds
Pod::Wordlist::Author::TABULO
Pod::Wordlist::Author::TABULOes
PodCoverage
PodCoverages
PodSyntaxTests
PodSyntaxTestses
PruneCruft
PruneCrufts
README
READMES
Raku
Rakus
Readme
ReadmeAnyFromPod
ReadmeAnyFromPods
Readmes
ReplaceName
ReplaceNames
RewriteVersion
RewriteVersions
RunExtraTests
RunExtraTestses
SeeAlso
SeeAlsoes
ShareDir
ShareDirs
SortSections
SortSectionses
SurgicalPodWeaver
SurgicalPodWeavers
TABULO
TABULOES
TABULON
TABULONS
Tabulo
TaskWeaver
TaskWeavers
TestRelease
TestReleases
UploadToCPAN
UploadToCPANs
Util
WarrantyDisclaimer
WarrantyDisclaimers
attr
attrs
bitbucket
bitbuckets
casefolding
casefoldings
cattr
cattrs
cpan
cpanfile
cpanfiles
cpans
darkpan
darkpans
dev
dir
dirs
dist
distro
distroes
distros
dists
dzil
dzils
evalbyte
evalbytes
fc
fcs
github
githubs
gitlab
gitlabs
lib
mayhap
mayhaps
merchantabilities
merchantability
optimization
optimizations
pattr
pattrs
repo
repoes
str
strs
trustme
trustmes
unsetting
unsettings
vc
vcs
wikidoc
wikidocs
xtag
xtags
zilla
zillas
